import click
from hub.log import logger
from hub.utils.store_control import StoreControlClient
from hub.utils.verify import Verify

@click.command()
@click.option('--username', '-u', default=None, help='Your AWS access key')
@click.option('--password', '-p', default=None, help='Your AWS secret key')
@click.option('--bucket',   '-b', default=None, help='Desired bucket name')
def configure(username, password, bucket):
    """ Logs in to Hub"""
    logger.info("Please log in using your AWS credentials.")
    if not username:
        logger.debug("Prompting for Access Key")
        username = click.prompt('AWS Access Key ID', type=str, hide_input=False)
    access_key = username.strip()

    if not password:
        logger.debug("Prompting for Secret Key")
        password = click.prompt('AWS Secret Access Key', type=str, hide_input=False)
    secret_key = password.strip()

    if not bucket:
        logger.debug("Prompting for bucket name")
        bucket = click.prompt('Bucket Name (e.g. company-name)', type=str, hide_input=False)
    bucket = bucket.strip()
    
    success, creds = Verify(access_key, secret_key).verify_aws(bucket)
    if success:
        StoreControlClient().save_config(creds)
        logger.info("Login Successful.")
    else:
        logger.error("Login error, please try again")
        
def logout():
    """ Logs out of Snark AI"""
    # TODO remove scripts
    TokenManager.purge_token()



from cloudvolume import CloudVolume

import warnings
from hub.utils.store_control import StoreControlClient

def _get_path(name):
    if len(name.split('/')) == 1:
        raise Exception('array name should be specified the following format (username/dataset:version)')
        return 
    user = name.split('/')[0]
    dataset = name.split('/')[1].split(':')[0]
    tag = name.split(':')
    if len(tag) == 1:
        tag.append('latest')
    tag = tag[1]
    bucket = StoreControlClient().get_config()['bucket']
    path = 's3://'+bucket+'/'+user+'/'+dataset+'/'+tag
    return path

def array(dim=None, name='username/test:latest', dtype='uint8'):
    path = _get_path(name)
    if not dim:
        return CloudVolume(path, parallel=True, progress=False, fill_missing=True, non_aligned_writes=True)

    return create(path, dim, dtype)
    
def create(path, dim=[50000, 28, 28], dtype='uint8', num_channels = 1):
    if len(dim)==1:
        dim = (dim[0],1,1,1)
        warnings.warn('hub arrays support only 3D/4D arrays. Expanding shape to {}'.format(str(dim)))

    if len(dim)==2:
        dim = (dim[0], dim[1], 1, 1)
        warnings.warn('hub arrays support only 3D/4D arrays. Expanding shape to {}'.format(str(dim)))

    if len(dim)==4:
        num_channels = dim[3]
        dim = dim[:3]
    
    if len(dim)>4:
        raise Exception('hub arrays support up to 4D')

    chunk_size = list(dim)
    chunk_size[0] = 1
    info = CloudVolume.create_new_info(
        num_channels    = num_channels,
        layer_type      = 'image',
        data_type       = dtype, # Channel images might be 'uint8'
        encoding        = 'raw', # raw, jpeg, compressed_segmentation, fpzip, kempressed
        resolution      = [1, 1, 1], # Voxel scaling, units are in nanometers
        voxel_offset    = [0, 0, 0], # x,y,z offset in voxels from the origin

        # Pick a convenient size for your underlying chunk representation
        # Powers of two are recommended, doesn't need to cover image exactly
        chunk_size      = chunk_size, # units are voxels
        volume_size     = dim, # e.g. a cubic millimeter dataset
    )

    vol = CloudVolume(path, info=info, parallel=True, fill_missing=True, progress=False,  non_aligned_writes=True, autocrop=True, green_threads=False)
    vol.commit_info()
    return vol

def load(name):
    path = _get_path(name)
    return CloudVolume(path, parallel=True, fill_missing=True, progress=False,  non_aligned_writes=True, autocrop=True, green_threads=False)
    
# TODO implement a cloudvolume simple wrapper that translates errors into hub errors
def delete(name):
    path = _get_path(name)
    bucket = StoreControlClient().get_config()['bucket']
    s3.Object(bucket, path.split(bucket)[-1]).delete()



import json
import hub.config as config
from hub.log import logger
import os

class StoreControlClient(object):
    """
    Controlling Snark Pods through rest api
    """
    def __init__(self):
        super(StoreControlClient, self).__init__()

    def is_authenticated(cls):
        return os.path.getsize(config.TOKEN_FILE_PATH) > 10

    def get_config(self):
        try:
            with open(config.TOKEN_FILE_PATH, 'r') as file:
                details = file.readlines()
                details = json.loads(''.join(details))

            with open(config.CLOUDVOLUME_PATH, 'r') as file:
                details = file.readlines()
                details = json.loads(''.join(details))
        except:
            logger.error("No AWS credentials found. Please configure credentials '> hub configure'")
            return {
                'AWS_ACCESS_KEY_ID':'',
                'AWS_SECRET_ACCESS_KEY': '',
                'bucket': ''
            }
        return details

    def save_config(self, details):
        try:
            os.mkdir('/'.join(config.TOKEN_FILE_PATH.split('/')[:-1]))
        except OSError:
            pass
        
        try:
            os.mkdir('/'.join(config.CLOUDVOLUME_PATH.split('/')[:-2]))
        except OSError:
            pass
            
        try:
            os.mkdir('/'.join(config.CLOUDVOLUME_PATH.split('/')[:-1]))
        except OSError:
            pass

        with open(config.TOKEN_FILE_PATH, 'w') as file:
            file.writelines(json.dumps(details))

        with open(config.CLOUDVOLUME_PATH, 'w') as file:
            file.writelines(json.dumps(details))

    def purge_token(self):
        if os.path.isfile(config.TOKEN_FILE_PATH):
            os.remove(config.TOKEN_FILE_PATH)
        if os.path.isfile(config.CLOUDVOLUME_PATH):
            os.remove(config.CLOUDVOLUME_PATH)